"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const __1 = require("..");
const path = require("path");
const Realm = require("realm");
const _ = require("lodash");
const faker = require("faker");
const del = require("del");
process.on('unhandledRejection', (reason, p) => {
    console.log('Unhandled Rejection at: Promise', p, 'reason:', reason);
});
const TickerSchema = {
    name: 'Ticker',
    primaryKey: 'tickerSymbol',
    properties: {
        'tickerSymbol': { type: 'string', optional: false, default: '' },
        'price': { type: 'float', optional: false, default: 0 },
        'companyName': { type: 'string', optional: false, default: '' }
    }
};
let server;
function main() {
    return __awaiter(this, void 0, void 0, function* () {
        yield del(path.resolve('./data'));
        server = new __1.Server({
            dataPath: path.resolve('./data'),
        });
        const authService = new __1.AuthService();
        authService.addProvider(new __1.auth.PasswordAuthProvider({ autoCreateAdminUser: true }));
        server.addService(authService);
        server.addService(new __1.RealmDirectoryService());
        server.addService(new __1.LogService());
        server.addService(new __1.SyncProxyService());
        server.addService(new __1.SyncService());
        server.addService(new __1.HealthService());
        const permissionsService = new __1.PermissionService();
        server.addService(permissionsService);
        yield server.start();
        const authServerUrl = `http://${server.address}`;
        const user = yield Realm.Sync.User.register(authServerUrl, 'max', 'ilovesushi');
        const adminToken = __1.generateAdminToken(server.privateKey);
        const adminTokenUser = Realm.Sync.User.adminUser(adminToken, authServerUrl);
        const realm = new Realm({
            sync: {
                user: user,
                url: `realm://127.0.0.1:9080/~/tickers`
            },
            schema: [TickerSchema]
        });
        Realm.Sync.setFeatureToken('eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJOb3RpZmllciI6dHJ1ZSwiU3luYyI6dHJ1ZSwiaWF0IjoxNDkyNjExMzEzfQ.Yb-o622ZYLKCQ0kTEDpBm6FsG68X9nj-OCdD4dT1G5W10fjhsTdUJiR1uYBilZPqgZEmIdDgd59rDpEjnqDhBCjTzpbsMVBGQrHj3B5eW4oGvxDkdqjR_fjwc5-zH3BeXMd9UAj13bLrxLsM0_g1FXIjmXCOK7YBrL0kKYAI2IVjqAGsI8zThhmbqxDvBeD8TVPc-brBZvXXHPO63ierJevjtZRz7mg_oV-B13gXXIxy2jlnevqFyBVRF6cd3bJmIpLUXHecQXqDGDeLaVlo2NDcnxl5OoVBN1QUSxptgQc-FIq-5qtxZXHooIJUpKQwtD2mBkJ-HBpxWbQR2C90UA');
        Realm.Sync.addListener(`realm://127.0.0.1:9080`, adminTokenUser, '^/.*?/tickers$', 'change', (e) => {
            console.log(e);
        });
        const TickerSymbolArray = ['RLM', 'APPL', 'MRK', 'MSFT', 'FB'];
        setInterval(() => {
            realm.write(() => {
                realm.create('Ticker', {
                    tickerSymbol: _.sample(TickerSymbolArray),
                    price: _.random(0, 100),
                    companyName: faker.company.companyName()
                }, true);
            });
        }, 2000);
        setInterval(() => {
            for (let r of permissionsService.adminRealm.objects('RealmFile')) {
                console.log(r);
            }
        }, 2000);
    });
}
main();
//# sourceMappingURL=permission-smoke-app.js.map