"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const RealmDefinition_1 = require("./RealmDefinition");
class UserMetadataRow {
}
UserMetadataRow.schema = {
    name: 'UserMetadataRow',
    properties: {
        user: { type: 'linkingObjects', objectType: 'User', property: 'metadata' },
        key: { type: 'string', optional: false },
        value: { type: 'string', optional: false }
    }
};
exports.UserMetadataRow = UserMetadataRow;
class User {
    constructor() {
        this.created = false;
    }
    toJSON() {
        return {
            user_id: this.userId,
            is_admin: this.isAdmin,
            accounts: this.accounts.slice().map((a) => a.toJSON()),
            metadata: this.metadata.slice(),
        };
    }
}
User.schema = {
    name: 'User',
    primaryKey: 'userId',
    properties: {
        userId: { type: 'string', optional: false },
        isAdmin: { type: 'bool', optional: false },
        accounts: { type: "list", objectType: "Account" },
        metadata: { type: 'list', objectType: 'UserMetadataRow', default: [], optional: false }
    }
};
exports.User = User;
class Account {
    toJSON() {
        return {
            provider: this.provider,
            provider_id: this.providerId,
        };
    }
}
Account.schema = {
    name: 'Account',
    properties: {
        provider: { type: 'string', optional: false, indexed: true },
        providerId: { type: "string", optional: false, indexed: true },
        user: { type: 'linkingObjects', objectType: 'User', property: 'accounts' }
    }
};
exports.Account = Account;
class Permission {
}
Permission.schema = {
    name: 'Permission',
    properties: {
        user: { type: 'User' },
        realmFile: 'RealmFile',
        mayRead: { type: 'bool', optional: false },
        mayWrite: { type: 'bool', optional: false },
        mayManage: { type: 'bool', optional: false },
        updatedAt: { type: 'date', optional: false },
    }
};
exports.Permission = Permission;
class RealmFile {
}
RealmFile.schema = {
    name: 'RealmFile',
    primaryKey: 'path',
    properties: {
        path: 'string',
        syncLabel: 'string',
        owner: { type: 'User', optional: true },
        createdAt: { type: 'date', optional: false },
        permissions: { type: 'list', objectType: 'Permission', default: [], optional: false }
    }
};
exports.RealmFile = RealmFile;
exports.AdminRealm = {
    remotePath: "/__admin",
    schema: [
        RealmDefinition_1.createRealmSchema(User),
        RealmDefinition_1.createRealmSchema(UserMetadataRow),
        RealmDefinition_1.createRealmSchema(Account),
        RealmDefinition_1.createRealmSchema(Permission),
        RealmDefinition_1.createRealmSchema(RealmFile),
    ],
};
//# sourceMappingURL=AdminRealm.js.map